@@ -1,4 +1,5 @@
 import logging
+from math import sqrt, ceil
 import random
 import Pyro4
 from pyage.core.agent.agent import AGENT
@@ -21,7 +22,7 @@ class Pyro4Locator(Locator):
         try:
             random_agent = self.__get_random_agent(agent)
             logger.debug(random_agent.get_address())
-            return  random_agent
+            return random_agent
         except:
             logging.exception('')
 
@@ -33,7 +34,7 @@ class Pyro4Locator(Locator):
         return Pyro4.Proxy(random.choice(agents.values()))
 
 
-class ParentLocator(Locator):
+class RandomLocator(Locator):
     def get_neighbour(self, agent):
         siblings = list(agent.parent.get_agents())
         if len(siblings) < 2:
@@ -42,9 +43,21 @@ class ParentLocator(Locator):
         return random.choice(siblings)
 
 
-class GridParentLocator(Locator):
+class RowLocator(Locator):
     def get_neighbour(self, agent):
         siblings = list(agent.parent.get_agents())
+        if len(siblings) < 2:
+            return None
         index = siblings.index(agent)
         return random.choice(siblings[index - 2:index] + siblings[index + 1:index + 3])
 
+
+class GridLocator(Locator):
+    def get_neighbour(self, agent):
+        siblings = list(agent.parent.get_agents())
+        if len(siblings) < 2:
+            return None
+        index = siblings.index(agent)
+        size = len(siblings)
+        dim = int(ceil(sqrt(size)))
+        return siblings[random.choice([index - dim, index - 1, index + 1, index + dim]) % size]
\ No newline at end of file
